package repositorio;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

import Iterators.IteratorCliente;
import bancoDeDados.Conexao;
import basicas.Cliente;
import interfaceRepositorio.IRepositorioCliente;

public class RepCliente implements IRepositorioCliente {
	private PreparedStatement prdStatement;
	private Conexao conexao;
	private ResultSet resultSet;

	private static final String INSERIR_CLIENTE_COM_FONE = "INSERT INTO cliente_tab ("
			+ "nome, cpf, array_telefones, email, nt_servicos)"
			+ "VALUES "
			+ "?, ?, LISTA_TELEFONES(?,?,?,?,?), " + "?, ?, nt_servicos() )";

	private static final String UPDATE_CLIENTE = "UPDATE cliente_tab SET "
			+ "nome = ?, cpf = ?, LISTA_TELEFONES = LISTA_TELEFONES(?,?,?,?,?), numeroCad = ?, email = ?, nt_servicos()"
			+ "WHERE numerodecadastro = ?";

	private static final String DELETE_CLIENTE = "DELETE FROM cliente_tab WHERE "
			+ "numerodecadastro = ?";

	private static final String SELECT_CLIENTE = "SELECT * FROM cliente_tab WHERE "
			+ "? ?";

	private static final String SELECT_TODOS_CLIENTES = "SELECT * FROM cliente_tab;";

	public RepCliente() throws ClassNotFoundException, SQLException {
		this.conexao = Conexao.getInstancia();
		this.prdStatement = null;
		this.resultSet = null;
	}

	public boolean cadastrar(Cliente cliente) throws SQLException {
		boolean retorno = false;

		this.prdStatement = this.conexao.getConnection().prepareStatement(
				INSERIR_CLIENTE_COM_FONE);
		this.prdStatement.setString(1, cliente.getNome());
		this.prdStatement.setString(2, cliente.getCpf());

		for (int i = 0; i < cliente.getTelefones().size(); i++) {
			if (cliente.getTelefones().size() <= i) {
				this.prdStatement.setString(3 + i, cliente.getTelefones()
						.get(i).getNumero());
			} else {
				this.prdStatement.setString(3 + i, null);
			}
		}

		this.prdStatement.setInt(8, cliente.getNumeroDeCadastro());
		this.prdStatement.setString(9, cliente.getEmail());

		retorno = this.prdStatement.execute();
		this.fecharTudo(this.prdStatement, this.resultSet);

		return retorno;
	}

	public boolean editar(Cliente cliente) throws SQLException {
		boolean retorno = false;

		this.prdStatement = this.conexao.getConnection().prepareStatement(
				UPDATE_CLIENTE);
		this.prdStatement.setString(1, cliente.getNome());
		this.prdStatement.setString(2, cliente.getCpf());

		for (int i = 0; i < cliente.getTelefones().size(); i++) {
			this.prdStatement.setString(3 + i, cliente.getTelefones().get(i)
					.getNumero());
		}

		this.prdStatement.setInt(8, cliente.getNumeroDeCadastro());
		this.prdStatement.setString(9, cliente.getEmail());
		this.prdStatement.setInt(10, cliente.getNumeroDeCadastro());

		retorno = this.prdStatement.execute();
		this.fecharTudo(this.prdStatement, this.resultSet);

		return retorno;
	}

	public boolean remover(Cliente cliente) throws SQLException {
		boolean retorno = false;

		this.prdStatement = this.conexao.getConnection().prepareStatement(
				DELETE_CLIENTE);
		this.prdStatement.setInt(1, cliente.getNumeroDeCadastro());

		retorno = this.prdStatement.execute();
		this.fecharTudo(prdStatement, resultSet);

		return retorno;
	}

	public IteratorCliente buscar(Cliente cliente) throws SQLException {

		this.prdStatement = this.conexao.getConnection().prepareStatement(
				SELECT_CLIENTE);
		this.prdStatement.setString(1, "numeroCad");
		this.prdStatement.setInt(2, cliente.getNumeroDeCadastro());
		this.resultSet = this.prdStatement.executeQuery();

		IteratorCliente retorno = new IteratorCliente(this.resultSet);
		this.fecharTudo(prdStatement, resultSet);
		return retorno;
	}

	private void fecharTudo(PreparedStatement prdSatemente, ResultSet resultSet)
			throws SQLException {
		prdSatemente.close();
		resultSet.close();
	}

	public void fechar() throws SQLException {
		Conexao.fechar(this.conexao.getConnection());
	}

	@Override
	public IteratorCliente buscarComParametros(String campoEscolhido,
			String valor) throws SQLException {

		this.prdStatement = this.conexao.getConnection().prepareStatement(
				SELECT_CLIENTE);
		this.prdStatement.setString(1, campoEscolhido);
		this.prdStatement.setString(2, valor);
		this.resultSet = this.prdStatement.executeQuery();

		return new IteratorCliente(this.resultSet);
	}

	@Override
	public IteratorCliente buscarTodos() throws SQLException {

		this.prdStatement = this.conexao.getConnection().prepareStatement(
				SELECT_TODOS_CLIENTES);
		this.resultSet = this.prdStatement.executeQuery();

		return new IteratorCliente(this.resultSet);
	}

	public static void main(String[] args) {
		RepCliente teste = null;
		try {
			// Class.forName("oracle.jdbc.driver.OracleDriver");
			teste = new RepCliente();
			System.out.println("TESTE");

			Cliente cliente = new Cliente("Maria Mariana Miranda", null, null,
					null, null);
			cliente.setNumeroDeCadastro(30);
		//	Cliente retorno = teste.buscar(cliente);

			teste.fechar();
			System.out.println("");
		} catch (ClassNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} finally {
			try {
				teste.fechar();
			} catch (SQLException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}

	}

}
