package repositorio;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

import Iterators.IteratorFuncionario;
import bancoDeDados.Conexao;
import basicas.Funcionario;
import interfaceRepositorio.IRepositorioFuncionario;

public class RepFuncionario implements IRepositorioFuncionario{
	private PreparedStatement prdStatement;
	private Conexao conexao;
	private ResultSet resultSet;

	private static final String INSERIR_FUNCIONARIO = "INSERT INTO funcionario_tab "
		+ "VALUES "
		+ "?, ?, LISTA_TELEFONES(?,?,?,?,?), "
		+ "(ENDERECO_TP(?, ?, ?, ?, ?, ?, ?, ?, ?))), ?, ?, ?, ?, ?, nt_servicos() )";

	private static final String UPDATE_FUNCIONARIO = "UPDATE funcionario_tab SET "
			+ "nome = ?, cpf = ?, LISTA_TELEFONES = LISTA_TELEFONES(?,?,?,?,?),"
			+ " ENDERECO_TP = (ENDERECO_TP(?, ?, ?, ?, ?, ?, ?, ?, ?))), Nacimento = ?, RG = ?, Cargo = ?, Salario = ?, Senha = ?, nt_servicos()"
			+ "WHERE numerodecadastro = ?";
	
	private static final String DELETE_FUNCIONARIO = "DELETE FROM funcionario_tab WHERE "
			+ "NumeroContrato = ?";
	
	private static final String SELECT_FUNCIONARIO = "SELECT * FROM funcionario_tab WHERE "
			+ "? ?";
	
	private static final String SELECT_TODOS_FUNCIONARIOS = "SELECT * FROM funcionario_tab";
	
	public RepFuncionario() throws ClassNotFoundException, SQLException {
		this.conexao = Conexao.getInstancia();
		this.prdStatement = null;
		this.resultSet = null;
	}
	
	public IteratorFuncionario buscar(Funcionario funcionario) throws SQLException {

		this.prdStatement = this.conexao.getConnection().prepareStatement(SELECT_FUNCIONARIO);
		this.prdStatement.setString(1, "NumeroDeCadastro");
		this.prdStatement.setInt(2, funcionario.getNumeroDeCadastro());
		this.resultSet = this.prdStatement.executeQuery();
		
		IteratorFuncionario retorno = new IteratorFuncionario(this.resultSet);
		this.fecharTudo(prdStatement, resultSet);
		return retorno;
	}

	public boolean cadastrar(Funcionario funcionario) throws SQLException {

		boolean retorno = false;

		this.prdStatement = this.conexao.getConnection().prepareStatement(
				INSERIR_FUNCIONARIO);
		this.prdStatement.setString(1, funcionario.getNome());
		this.prdStatement.setString(2, funcionario.getCpf());

		for (int i = 0; i < 5; i++) {
			if(funcionario.getTelefones().size() <= i){
				this.prdStatement.setString(3 + i, funcionario.getTelefones().get(i).getNumero());				
			}else{
				this.prdStatement.setString(3 + i, null);
			}	
		}

		this.prdStatement.setString(9, funcionario.getEndereco().getPais());
		this.prdStatement.setString(10, funcionario.getEndereco().getEstado());
		this.prdStatement.setString(11, funcionario.getEndereco().getCidade());
		this.prdStatement.setString(12, funcionario.getEndereco().getBairro());
		this.prdStatement.setString(13, funcionario.getEndereco().getRua());
		this.prdStatement.setString(14, funcionario.getEndereco().getNumero());
		this.prdStatement.setString(15, funcionario.getEndereco().getComplemento());
		this.prdStatement.setString(16, funcionario.getEndereco().getCep());
		this.prdStatement.setString(17, funcionario.getDataDeNascimento());
		this.prdStatement.setString(18, funcionario.getRg());
		this.prdStatement.setString(19, funcionario.getCargo());
		this.prdStatement.setDouble(20, funcionario.getSalario());
		this.prdStatement.setString(21, funcionario.getSenha());

		retorno = this.prdStatement.execute();
		this.fecharTudo(this.prdStatement, this.resultSet);

		return retorno;
	}

	public boolean editar(Funcionario funcionario) throws SQLException {
		boolean retorno = false;

		this.prdStatement = this.conexao.getConnection().prepareStatement(
				UPDATE_FUNCIONARIO);
		this.prdStatement.setString(1, funcionario.getNome());
		this.prdStatement.setString(2, funcionario.getCpf());

		for (int i = 0; i < 5; i++) {
			if(funcionario.getTelefones().size() <= i){
				this.prdStatement.setString(3 + i, funcionario.getTelefones().get(i).getNumero());				
			}else{
				this.prdStatement.setString(3 + i, null);
			}	
		}

		this.prdStatement.setString(9, funcionario.getEndereco().getPais());
		this.prdStatement.setString(10, funcionario.getEndereco().getEstado());
		this.prdStatement.setString(11, funcionario.getEndereco().getCidade());
		this.prdStatement.setString(12, funcionario.getEndereco().getBairro());
		this.prdStatement.setString(13, funcionario.getEndereco().getRua());
		this.prdStatement.setString(14, funcionario.getEndereco().getNumero());
		this.prdStatement.setString(15, funcionario.getEndereco().getComplemento());
		this.prdStatement.setString(16, funcionario.getEndereco().getCep());
		this.prdStatement.setString(17, funcionario.getDataDeNascimento());
		this.prdStatement.setString(18, funcionario.getRg());
		this.prdStatement.setString(19, funcionario.getCargo());
		this.prdStatement.setDouble(20, funcionario.getSalario());
		this.prdStatement.setString(21, funcionario.getSenha());

		retorno = this.prdStatement.execute();
		this.fecharTudo(this.prdStatement, this.resultSet);

		return retorno;
	}

	public boolean remover(Funcionario funcionario) throws SQLException {
		boolean retorno = false;

		this.prdStatement = this.conexao.getConnection().prepareStatement(
				DELETE_FUNCIONARIO);
		this.prdStatement.setInt(1, funcionario.getNumeroDeCadastro());

		retorno = this.prdStatement.execute();
		this.fecharTudo(prdStatement, resultSet);

		return retorno;
	}

	public IteratorFuncionario buscarComParametros(String campoEscolhido, String valor) throws SQLException {
	
		this.prdStatement = this.conexao.getConnection().prepareStatement(SELECT_FUNCIONARIO);
		this.prdStatement.setString(1, campoEscolhido);
		this.prdStatement.setString(2, valor);
		this.resultSet = this.prdStatement.executeQuery();

		IteratorFuncionario retorno = new IteratorFuncionario(this.resultSet);
		this.fecharTudo(prdStatement, resultSet);
		return retorno;
	}

	public void fechar() throws SQLException {
		Conexao.fechar(this.conexao.getConnection());
	}
	
	private void fecharTudo(PreparedStatement prdSatemente, ResultSet resultSet)throws SQLException {
		prdSatemente.close();
		resultSet.close();
	}
	
	public IteratorFuncionario buscarTodos() throws SQLException {

		this.prdStatement = this.conexao.getConnection().prepareStatement(SELECT_TODOS_FUNCIONARIOS);
		this.resultSet = this.prdStatement.executeQuery();
		
		IteratorFuncionario retorno = new IteratorFuncionario(this.resultSet);
		this.fecharTudo(prdStatement, resultSet);
		return retorno;
	}

}
