package repositorio;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

import Iterators.IteratorVeiculo;
import bancoDeDados.Conexao;
import basicas.Veiculo;
import interfaceRepositorio.IRepositorioVeiculo;

public class RepVeiculo implements IRepositorioVeiculo {
	private PreparedStatement prdStatement;
	private Conexao conexao;
	private ResultSet resultSet;
	
	private static final String INSERIR_VEICULO = "INSERT INTO veiculo_tab " 
		+ "VALUES "
		+ "?, ?, ?, ?, Modelo_tp(?,?)";
	
	private static final String UPDATE_VEICULO = "UPDATE veiculo_tab SET " 
		+ "ano = ?, cor = ?, placa = ?, chassi = ?, Modelo_tp = Modelo_tp(?,?)";
	
	private static final String DELETE_VEICULO = "DELETE FROM veiculo_tab WHERE "
		+ "numeroDeCadastro = ?";
	
	private static final String SELECT_VEICULO = "SELECT * FROM veiculo_tab WHERE "
		+ "? ?";
	
	private static final String SELECT_TODOS_VEICULO = "SELECT * FROM veiculo_tab";
	
	public RepVeiculo() throws ClassNotFoundException, SQLException {
		this.conexao = Conexao.getInstancia();
		this.prdStatement = null;
		this.resultSet = null;
	}
	
	public IteratorVeiculo buscar(Veiculo veiculo) throws SQLException {

		this.prdStatement = this.conexao.getConnection().prepareStatement(SELECT_VEICULO);
		this.prdStatement.setString(1, "numeroDeCadastro");
		this.prdStatement.setInt(2, veiculo.getNumeroDeCadastro());
		this.resultSet = this.prdStatement.executeQuery();

		IteratorVeiculo retorno = new IteratorVeiculo(this.resultSet);
		this.fecharTudo(prdStatement, resultSet);
		return retorno;
	}

	public boolean cadastrar(Veiculo veiculo) throws SQLException {
		boolean retorno = false;
		
		this.prdStatement = this.conexao.getConnection().prepareStatement(INSERIR_VEICULO);
		this.prdStatement.setString(1, veiculo.getAno());
		this.prdStatement.setString(2, veiculo.getCor());
		this.prdStatement.setString(3, veiculo.getPlaca());
		this.prdStatement.setString(4, veiculo.getChassi());
		this.prdStatement.setString(5, veiculo.getModelo().getMarca());
		this.prdStatement.setString(6, veiculo.getModelo().getModelo());
	
		retorno = this.prdStatement.execute();
		this.fecharTudo(this.prdStatement, this.resultSet);

		return retorno;
	}

	public boolean editar(Veiculo veiculo) throws SQLException {
		boolean retorno = false;
		
		this.prdStatement = this.conexao.getConnection().prepareStatement(UPDATE_VEICULO);
		this.prdStatement.setString(1, veiculo.getAno());
		this.prdStatement.setString(2, veiculo.getCor());
		this.prdStatement.setString(3, veiculo.getPlaca());
		this.prdStatement.setString(4, veiculo.getChassi());
		this.prdStatement.setString(5, veiculo.getModelo().getMarca());
		this.prdStatement.setString(6, veiculo.getModelo().getModelo());
	
		retorno = this.prdStatement.execute();
		this.fecharTudo(this.prdStatement, this.resultSet);

		return retorno;
	}

	public boolean remover(Veiculo veiculo) throws SQLException {
		boolean retorno = false;

		this.prdStatement = this.conexao.getConnection().prepareStatement(DELETE_VEICULO);
		this.prdStatement.setInt(1, veiculo.getNumeroDeCadastro());

		retorno = this.prdStatement.execute();
		this.fecharTudo(prdStatement, resultSet);

		return retorno;
	}
	
	private void fecharTudo(PreparedStatement prdSatemente, ResultSet resultSet)throws SQLException {
		prdSatemente.close();
		resultSet.close();
	}

	@Override
	public IteratorVeiculo buscarComParametros(String campoEscolhido, String valor) throws SQLException {
		this.prdStatement = this.conexao.getConnection().prepareStatement(SELECT_VEICULO);
		this.prdStatement.setString(1, campoEscolhido);
		this.prdStatement.setString(2, valor);
		this.resultSet = this.prdStatement.executeQuery();

		IteratorVeiculo retorno = new IteratorVeiculo(this.resultSet);
		this.fecharTudo(prdStatement, resultSet);
		return retorno;
	}

	@Override
	public IteratorVeiculo buscarTodos() throws SQLException {
		this.prdStatement = this.conexao.getConnection().prepareStatement(SELECT_TODOS_VEICULO);
		this.resultSet = this.prdStatement.executeQuery();

		IteratorVeiculo retorno = new IteratorVeiculo(this.resultSet);
		this.fecharTudo(prdStatement, resultSet);
		return retorno;
	}

}
